@@ -139,30 +139,7 @@ function needHomepageOverride(prefb) {
  * @return The override page.
  */
 function getPostUpdateOverridePage(defaultOverridePage) {
-  var um = Components.classes["@mozilla.org/updates/update-manager;1"]
-                     .getService(Components.interfaces.nsIUpdateManager);
-  try {
-    // If the updates.xml file is deleted then getUpdateAt will throw.
-    var update = um.getUpdateAt(0)
-                   .QueryInterface(Components.interfaces.nsIPropertyBag);
-  } catch (e) {
-    // This should never happen.
-    Components.utils.reportError("Unable to find update: " + e);
-    return defaultOverridePage;
-  }
-
-  let actions = update.getProperty("actions");
-  // When the update doesn't specify actions fallback to the original behavior
-  // of displaying the default override page.
-  if (!actions)
-    return defaultOverridePage;
-
-  // The existence of silent or the non-existence of showURL in the actions both
-  // mean that an override page should not be displayed.
-  if (actions.indexOf("silent") != -1 || actions.indexOf("showURL") == -1)
-    return "";
-
-  return update.getProperty("openURL") || defaultOverridePage;
+  return ""; // disabled by bug 792181
 }
 
 // Copies a pref override file into the user's profile pref-override folder,
@@ -245,7 +222,7 @@ function openWindow(parent, url, target, features, args, noExternalArgs) {
   }
 
   // Pass these as null to ensure that we always trigger the "single URL"
-  // behavior in browser.js's BrowserStartup (which handles the window
+  // behavior in browser.js's gBrowserInit.onLoad (which handles the window
   // arguments)
   argArray.AppendElement(null); // charset
   argArray.AppendElement(null); // referer
