@@ -70,7 +70,8 @@ IndexedDBParent::RecvPIndexedDBDatabaseConstructor(
 {
   nsRefPtr<IDBOpenDBRequest> request;
   nsresult rv =
-    mFactory->OpenCommon(aName, aVersion, false, getter_AddRefs(request));
+    mFactory->OpenCommon(aName, aVersion, false, nsnull,
+                         getter_AddRefs(request));
   NS_ENSURE_SUCCESS(rv, false);
 
   IndexedDBDatabaseParent* actor =
@@ -93,7 +94,7 @@ IndexedDBParent::RecvPIndexedDBDeleteDatabaseRequestConstructor(
   nsRefPtr<IDBOpenDBRequest> request;
 
   nsresult rv =
-    mFactory->OpenCommon(aName, 0, true, getter_AddRefs(request));
+    mFactory->OpenCommon(aName, 0, true, nsnull, getter_AddRefs(request));
   NS_ENSURE_SUCCESS(rv, false);
 
   rv = actor->SetOpenRequest(request);
@@ -377,7 +378,7 @@ IndexedDBDatabaseParent::HandleDatabaseEvent(nsIDOMEvent* aEvent,
   nsresult rv;
 
   if (aType.EqualsLiteral(VERSIONCHANGE_EVT_STR)) {
-    JSContext* cx = nsContentUtils::GetCurrentJSContext();
+    JSContext* cx = nsContentUtils::GetSafeJSContext();
     NS_ENSURE_TRUE(cx, NS_ERROR_FAILURE);
 
     nsCOMPtr<nsIIDBVersionChangeEvent> changeEvent = do_QueryInterface(aEvent);
@@ -444,9 +445,6 @@ IndexedDBDatabaseParent::RecvPIndexedDBTransactionConstructor(
 
   const NormalTransactionParams& params = aParams.get_NormalTransactionParams();
 
-  const InfallibleTArray<nsString>& names = params.names();
-  const IDBTransaction::Mode& mode = params.mode();
-
   nsTArray<nsString> storesToOpen;
   storesToOpen.AppendElements(params.names());
 
@@ -580,7 +578,7 @@ bool
 IndexedDBTransactionParent::RecvAbort(const nsresult& aAbortCode)
 {
   MOZ_ASSERT(mTransaction);
-  mTransaction->AbortWithCode(aAbortCode);
+  mTransaction->Abort(aAbortCode);
   return true;
 }
 
@@ -963,9 +961,6 @@ IndexedDBVersionChangeObjectStoreParent::RecvPIndexedDBIndexConstructor(
     const CreateIndexParams& params = aParams.get_CreateIndexParams();
     const IndexInfo& info = params.info();
 
-    // Copy...
-    nsTArray<nsString> keyPathArray = info.keyPathArray;
-
     nsRefPtr<IDBIndex> index;
 
     nsresult rv;
@@ -973,8 +968,7 @@ IndexedDBVersionChangeObjectStoreParent::RecvPIndexedDBIndexConstructor(
     {
       AutoSetCurrentTransaction asct(mObjectStore->Transaction());
 
-      rv = mObjectStore->CreateIndexInternal(info, keyPathArray,
-                                             getter_AddRefs(index));
+      rv = mObjectStore->CreateIndexInternal(info, getter_AddRefs(index));
     }
 
     NS_ENSURE_SUCCESS(rv, false);
@@ -1203,7 +1197,8 @@ IndexedDBObjectStoreRequestParent::Get(const GetParams& aParams)
   {
     AutoSetCurrentTransaction asct(mObjectStore->Transaction());
 
-    nsresult rv = mObjectStore->GetInternal(keyRange, getter_AddRefs(request));
+    nsresult rv = mObjectStore->GetInternal(keyRange, nsnull,
+                                            getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
 
@@ -1242,6 +1237,7 @@ IndexedDBObjectStoreRequestParent::GetAll(const GetAllParams& aParams)
     AutoSetCurrentTransaction asct(mObjectStore->Transaction());
 
     nsresult rv = mObjectStore->GetAllInternal(keyRange, aParams.limit(),
+                                               nsnull,
                                                getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
@@ -1314,7 +1310,7 @@ IndexedDBObjectStoreRequestParent::Delete(const DeleteParams& aParams)
     AutoSetCurrentTransaction asct(mObjectStore->Transaction());
 
     nsresult rv =
-      mObjectStore->DeleteInternal(keyRange, getter_AddRefs(request));
+      mObjectStore->DeleteInternal(keyRange, nsnull, getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
 
@@ -1333,7 +1329,7 @@ IndexedDBObjectStoreRequestParent::Clear(const ClearParams& aParams)
   {
     AutoSetCurrentTransaction asct(mObjectStore->Transaction());
 
-    nsresult rv = mObjectStore->ClearInternal(getter_AddRefs(request));
+    nsresult rv = mObjectStore->ClearInternal(nsnull, getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
 
@@ -1372,7 +1368,7 @@ IndexedDBObjectStoreRequestParent::Count(const CountParams& aParams)
     AutoSetCurrentTransaction asct(mObjectStore->Transaction());
 
     nsresult rv =
-      mObjectStore->CountInternal(keyRange, getter_AddRefs(request));
+      mObjectStore->CountInternal(keyRange, nsnull, getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
 
@@ -1413,7 +1409,7 @@ IndexedDBObjectStoreRequestParent::OpenCursor(const OpenCursorParams& aParams)
     AutoSetCurrentTransaction asct(mObjectStore->Transaction());
 
     nsresult rv =
-      mObjectStore->OpenCursorInternal(keyRange, direction,
+      mObjectStore->OpenCursorInternal(keyRange, direction, nsnull,
                                        getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
@@ -1457,7 +1453,8 @@ IndexedDBIndexRequestParent::Get(const GetParams& aParams)
   {
     AutoSetCurrentTransaction asct(mIndex->ObjectStore()->Transaction());
 
-    nsresult rv = mIndex->GetInternal(keyRange, getter_AddRefs(request));
+    nsresult rv = mIndex->GetInternal(keyRange, nsnull,
+                                      getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
 
@@ -1480,7 +1477,8 @@ IndexedDBIndexRequestParent::GetKey(const GetKeyParams& aParams)
   {
     AutoSetCurrentTransaction asct(mIndex->ObjectStore()->Transaction());
 
-    nsresult rv = mIndex->GetKeyInternal(keyRange, getter_AddRefs(request));
+    nsresult rv = mIndex->GetKeyInternal(keyRange, nsnull,
+                                         getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
 
@@ -1518,7 +1516,7 @@ IndexedDBIndexRequestParent::GetAll(const GetAllParams& aParams)
   {
     AutoSetCurrentTransaction asct(mIndex->ObjectStore()->Transaction());
 
-    nsresult rv = mIndex->GetAllInternal(keyRange, aParams.limit(),
+    nsresult rv = mIndex->GetAllInternal(keyRange, aParams.limit(), nsnull,
                                          getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
@@ -1557,7 +1555,7 @@ IndexedDBIndexRequestParent::GetAllKeys(const GetAllKeysParams& aParams)
   {
     AutoSetCurrentTransaction asct(mIndex->ObjectStore()->Transaction());
 
-    nsresult rv = mIndex->GetAllKeysInternal(keyRange, aParams.limit(),
+    nsresult rv = mIndex->GetAllKeysInternal(keyRange, aParams.limit(), nsnull,
                                              getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
@@ -1596,7 +1594,8 @@ IndexedDBIndexRequestParent::Count(const CountParams& aParams)
   {
     AutoSetCurrentTransaction asct(mIndex->ObjectStore()->Transaction());
 
-    nsresult rv = mIndex->CountInternal(keyRange, getter_AddRefs(request));
+    nsresult rv = mIndex->CountInternal(keyRange, nsnull,
+                                        getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
 
@@ -1637,7 +1636,8 @@ IndexedDBIndexRequestParent::OpenCursor(const OpenCursorParams& aParams)
     AutoSetCurrentTransaction asct(mIndex->ObjectStore()->Transaction());
 
     nsresult rv =
-      mIndex->OpenCursorInternal(keyRange, direction, getter_AddRefs(request));
+      mIndex->OpenCursorInternal(keyRange, direction, nsnull,
+                                 getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
 
@@ -1678,7 +1678,7 @@ IndexedDBIndexRequestParent::OpenKeyCursor(const OpenKeyCursorParams& aParams)
     AutoSetCurrentTransaction asct(mIndex->ObjectStore()->Transaction());
 
     nsresult rv =
-      mIndex->OpenKeyCursorInternal(keyRange, direction,
+      mIndex->OpenKeyCursorInternal(keyRange, direction, nsnull,
                                     getter_AddRefs(request));
     NS_ENSURE_SUCCESS(rv, false);
   }
