@@ -8,7 +8,7 @@
 
 #if defined(XP_UNIX)
 
-#ifdef MOZ_WIDGET_GTK2
+#ifdef MOZ_WIDGET_GTK
 #include <gdk/gdkx.h>
 // we're using default display for now
 #define GET_NATIVE_WINDOW(aWidget) (EGLNativeWindowType)GDK_WINDOW_XID((GdkWindow *) aWidget->GetNativeData(NS_NATIVE_WINDOW))
@@ -35,6 +35,19 @@
 #include "nsSurfaceTexture.h"
 #endif
 #include <android/log.h>
+#define LOG(args...)  __android_log_print(ANDROID_LOG_INFO, "Gonk" , ## args)
+
+# if defined(MOZ_WIDGET_GONK)
+#  include "cutils/properties.h"
+#  include <ui/GraphicBuffer.h>
+
+using namespace android;
+
+# define EGL_NATIVE_BUFFER_ANDROID 0x3140
+# define EGL_IMAGE_PRESERVED_KHR   0x30D2
+
+# endif
+
 #endif
 
 #define GLES2_LIB "libGLESv2.so"
@@ -42,7 +55,7 @@
 
 #elif defined(XP_WIN)
 
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 
 #define GLES2_LIB "libGLESv2.dll"
 
@@ -108,7 +121,8 @@ public:
 
 #include "gfxCrashReporterUtils.h"
 
-#ifdef MOZ_PLATFORM_MAEMO
+
+#if defined(MOZ_PLATFORM_MAEMO) || defined(MOZ_WIDGET_GONK)
 static bool gUseBackingSurface = true;
 #else
 static bool gUseBackingSurface = false;
@@ -132,8 +146,11 @@ static GLLibraryEGL sEGLLibrary;
     (_array).AppendElement(_k);                 \
 } while (0)
 
+#ifndef MOZ_JAVA_COMPOSITOR
 static EGLSurface
 CreateSurfaceForWindow(nsIWidget *aWidget, EGLConfig config);
+#endif
+
 static bool
 CreateConfig(EGLConfig* aConfig);
 #ifdef MOZ_X11
@@ -293,6 +310,14 @@ public:
 #endif
             }
 
+#ifdef MOZ_WIDGET_GONK
+        gUseBackingSurface = Preferences::GetBool("gfx.gralloc.enabled", false);
+        char propValue[PROPERTY_VALUE_MAX];
+        property_get("ro.build.version.sdk", propValue, "0");
+        if (atoi(propValue) < 15)
+            gUseBackingSurface = false;
+#endif
+
         bool current = MakeCurrent();
         if (!current) {
             gfx::LogFailure(NS_LITERAL_CSTRING(
@@ -304,20 +329,11 @@ public:
 
         PR_STATIC_ASSERT(sizeof(GLint) >= sizeof(int32_t));
         mMaxTextureImageSize = PR_INT32_MAX;
-#if 0
-        if (ok) {
-            EGLint v;
-            sEGLLibrary.fQueryContext(EGL_DISPLAY(), mContext, LOCAL_EGL_RENDER_BUFFER, &v);
-            if (v == LOCAL_EGL_BACK_BUFFER)
-                mIsDoubleBuffered = true;
-        }
-#endif
 
         mShareWithEGLImage = sEGLLibrary.HasKHRImageBase() &&
                              sEGLLibrary.HasKHRImageTexture2D() &&
                              IsExtensionSupported(OES_EGL_image);
 
-
         if (ok)
             InitFramebuffers();
 
@@ -857,8 +873,6 @@ void GLContextEGL::ReleaseSharedHandle(TextureImage::TextureShareType aType,
         return;
     }
 
-    NS_ASSERTION(mShareWithEGLImage, "EGLImage not supported or disabled in runtime");
-
     SharedTextureHandleWrapper* wrapper = reinterpret_cast<SharedTextureHandleWrapper*>(aSharedHandle);
 
     switch (wrapper->Type()) {
@@ -887,6 +901,7 @@ void GLContextEGL::ReleaseSharedHandle(TextureImage::TextureShareType aType,
         delete wrap;
         break;
     }
+    
     default:
         NS_ERROR("Unknown shared handle type");
     }
@@ -958,7 +973,7 @@ bool GLContextEGL::AttachSharedHandle(TextureImage::TextureShareType aType,
         NS_ASSERTION(mShareWithEGLImage, "EGLImage not supported or disabled in runtime");
 
         EGLTextureWrapper* wrap = (EGLTextureWrapper*)aSharedHandle;
-        fImageTargetTexture2D(LOCAL_GL_TEXTURE_2D, wrap->GetEGLImage());
+        sEGLLibrary.fImageTargetTexture2DOES(LOCAL_GL_TEXTURE_2D, wrap->GetEGLImage());
         break;
     }
 
@@ -1083,18 +1098,41 @@ GLFormatForImage(gfxASurface::gfxImageFormat aFormat)
         return LOCAL_GL_RGBA;
     case gfxASurface::ImageFormatRGB16_565:
         return LOCAL_GL_RGB;
+    case gfxASurface::ImageFormatA8:
+        return LOCAL_GL_LUMINANCE;
     default:
         NS_WARNING("Unknown GL format for Image format");
     }
     return 0;
 }
 
+#ifdef MOZ_WIDGET_GONK
+static PixelFormat
+PixelFormatForImage(gfxASurface::gfxImageFormat aFormat)
+{
+    switch (aFormat) {
+    case gfxASurface::ImageFormatARGB32:
+        return PIXEL_FORMAT_RGBA_8888;
+    case gfxASurface::ImageFormatRGB24:
+        return PIXEL_FORMAT_RGBX_8888;
+    case gfxASurface::ImageFormatRGB16_565:
+        return PIXEL_FORMAT_RGB_565;
+    case gfxASurface::ImageFormatA8:
+        return PIXEL_FORMAT_L_8;
+    default:
+        MOZ_NOT_REACHED("Unknown gralloc pixel format for Image format");
+    }
+    return 0;
+}
+#endif
+
 static GLenum
 GLTypeForImage(gfxASurface::gfxImageFormat aFormat)
 {
     switch (aFormat) {
     case gfxASurface::ImageFormatARGB32:
     case gfxASurface::ImageFormatRGB24:
+    case gfxASurface::ImageFormatA8:
         return LOCAL_GL_UNSIGNED_BYTE;
     case gfxASurface::ImageFormatRGB16_565:
         return LOCAL_GL_UNSIGNED_SHORT_5_6_5;
@@ -1117,10 +1155,10 @@ public:
         : TextureImage(aSize, aWrapMode, aContentType, aFlags)
         , mGLContext(aContext)
         , mUpdateFormat(gfxASurface::ImageFormatUnknown)
+        , mEGLImage(nsnull)
+        , mTexture(aTexture)
         , mSurface(nsnull)
         , mConfig(nsnull)
-        , mTexture(aTexture)
-        , mEGLImage(nsnull)
         , mTextureState(Created)
         , mBound(false)
         , mIsLocked(false)
@@ -1128,11 +1166,28 @@ public:
         mUpdateFormat = gfxPlatform::GetPlatform()->OptimalFormatForContent(GetContentType());
 
         if (gUseBackingSurface) {
+#ifdef MOZ_WIDGET_GONK
+            switch (mUpdateFormat) {
+            case gfxASurface::ImageFormatARGB32:
+                mShaderType = BGRALayerProgramType;
+                break;
+            case gfxASurface::ImageFormatRGB24:
+                mUpdateFormat = gfxASurface::ImageFormatARGB32;
+                mShaderType = BGRXLayerProgramType;
+                break;
+            case gfxASurface::ImageFormatRGB16_565:
+                mShaderType = RGBXLayerProgramType;
+                break;
+            default:
+                MOZ_NOT_REACHED("Unknown update format");
+            }
+#else
             if (mUpdateFormat != gfxASurface::ImageFormatARGB32) {
                 mShaderType = RGBXLayerProgramType;
             } else {
                 mShaderType = RGBALayerProgramType;
             }
+#endif
             Resize(aSize);
         } else {
             if (mUpdateFormat == gfxASurface::ImageFormatRGB16_565) {
@@ -1167,19 +1222,32 @@ public:
         }
     }
 
+    bool UsingDirectTexture()
+    {
+#ifdef MOZ_WIDGET_GONK
+        if (mGraphicBuffer != nsnull)
+            return true;
+#endif
+        return !!mBackingSurface;
+    }
+
     virtual void GetUpdateRegion(nsIntRegion& aForRegion)
     {
         if (mTextureState != Valid) {
             // if the texture hasn't been initialized yet, force the
             // client to paint everything
             aForRegion = nsIntRect(nsIntPoint(0, 0), mSize);
-        } else if (!mBackingSurface) {
-            // We can only draw a rectangle, not subregions due to
-            // the way that our texture upload functions work.  If
-            // needed, we /could/ do multiple texture uploads if we have
-            // non-overlapping rects, but that's a tradeoff.
-            aForRegion = nsIntRegion(aForRegion.GetBounds());
         }
+
+        if (UsingDirectTexture()) {
+            return;
+        }
+
+        // We can only draw a rectangle, not subregions due to
+        // the way that our texture upload functions work.  If
+        // needed, we /could/ do multiple texture uploads if we have
+        // non-overlapping rects, but that's a tradeoff.
+        aForRegion = nsIntRegion(aForRegion.GetBounds());
     }
 
     virtual gfxASurface* BeginUpdate(nsIntRegion& aRegion)
@@ -1196,6 +1264,14 @@ public:
             return NULL;
         }
 
+#ifdef MOZ_WIDGET_GONK
+        if (mGraphicBuffer != nsnull) {
+            mUpdateSurface = GetLockSurface();
+
+            return mUpdateSurface;
+        }
+#endif
+
         if (mBackingSurface) {
             if (sEGLLibrary.HasKHRLockSurface()) {
                 mUpdateSurface = GetLockSurface();
@@ -1311,7 +1387,11 @@ public:
             region = aRegion;
         }
 
-        if (mBackingSurface && sEGLLibrary.HasKHRLockSurface()) {
+        if ((mBackingSurface && sEGLLibrary.HasKHRLockSurface())
+#ifdef MOZ_WIDGET_GONK
+            || (mGraphicBuffer != nsnull)
+#endif
+            ) {
             mUpdateSurface = GetLockSurface();
             if (mUpdateSurface) {
                 nsRefPtr<gfxContext> ctx = new gfxContext(mUpdateSurface);
@@ -1343,9 +1423,21 @@ public:
             Resize(mSize);
         }
 
-        mGLContext->fActiveTexture(aTextureUnit);
-        mGLContext->fBindTexture(LOCAL_GL_TEXTURE_2D, mTexture);
-        mGLContext->fActiveTexture(LOCAL_GL_TEXTURE0);
+#ifdef MOZ_WIDGET_GONK
+        if (UsingDirectTexture()) {
+            mGLContext->fActiveTexture(aTextureUnit);
+            mGLContext->fBindTexture(LOCAL_GL_TEXTURE_2D, mTexture);
+            sEGLLibrary.fImageTargetTexture2DOES(LOCAL_GL_TEXTURE_2D, mEGLImage);
+            if (sEGLLibrary.fGetError() != LOCAL_EGL_SUCCESS) {
+               LOG("Could not set image target texture. ERROR (0x%04x)", sEGLLibrary.fGetError());
+            }
+        } else
+#endif
+        {
+            mGLContext->fActiveTexture(aTextureUnit);
+            mGLContext->fBindTexture(LOCAL_GL_TEXTURE_2D, mTexture);
+            mGLContext->fActiveTexture(LOCAL_GL_TEXTURE0);
+        }
     }
 
     virtual GLuint GetTextureID() 
@@ -1367,13 +1459,13 @@ public:
             return;
 
         mGLContext->fBindTexture(LOCAL_GL_TEXTURE_2D, mTexture);
-	
+    
         // Try to generate a backin surface first if we have the ability
         if (gUseBackingSurface) {
             CreateBackingSurface(gfxIntSize(aSize.width, aSize.height));
         }
 
-        if (!mBackingSurface) {
+        if (!UsingDirectTexture()) {
             // If we don't have a backing surface or failed to obtain one,
             // use the GL Texture failsafe
             mGLContext->fTexImage2D(LOCAL_GL_TEXTURE_2D,
@@ -1432,6 +1524,38 @@ public:
             return nsnull;
         }
 
+#ifdef MOZ_WIDGET_GONK
+        if (mGraphicBuffer != nsnull) {
+            // Unset the EGLImage target so that we don't get clashing locks
+            mGLContext->MakeCurrent(true);
+            mGLContext->fActiveTexture(LOCAL_GL_TEXTURE0);
+            mGLContext->fBindTexture(LOCAL_GL_TEXTURE_2D, mTexture);
+            sEGLLibrary.fImageTargetTexture2DOES(LOCAL_GL_TEXTURE_2D, 0);
+            if (sEGLLibrary.fGetError() != LOCAL_EGL_SUCCESS) {
+                LOG("Could not set image target texture. ERROR (0x%04x)", sEGLLibrary.fGetError());
+                return false;
+            }
+
+            void *vaddr;
+            if (mGraphicBuffer->lock(GraphicBuffer::USAGE_SW_READ_OFTEN |
+                                     GraphicBuffer::USAGE_SW_WRITE_OFTEN,
+                                     &vaddr) != OK) {
+                LOG("Could not lock GraphicBuffer");
+                return false;
+            }
+
+            nsRefPtr<gfxImageSurface> surface =
+                new gfxImageSurface(reinterpret_cast<unsigned char *>(vaddr),
+                                    gfxIntSize(mSize.width, mSize.height),
+                                    mGraphicBuffer->getStride() * gfxUtils::ImageFormatToDepth(mUpdateFormat) / 8,
+                                    mUpdateFormat);
+
+            mIsLocked = true;
+
+            return surface.forget();
+        }
+#endif
+
         if (!sEGLLibrary.HasKHRLockSurface()) {
             NS_WARNING("GetLockSurface called, but no EGL_KHR_lock_surface extension!");
             return nsnull;
@@ -1476,8 +1600,17 @@ public:
             return;
         }
 
-        sEGLLibrary.fUnlockSurface(EGL_DISPLAY(), mSurface);
         mIsLocked = false;
+
+#ifdef MOZ_WIDGET_GONK
+        if (mGraphicBuffer != nsnull) {
+            mGraphicBuffer->unlock();
+
+            return;
+        }
+#endif
+
+        sEGLLibrary.fUnlockSurface(EGL_DISPLAY(), mSurface);
     }
 
     virtual already_AddRefed<gfxASurface> GetBackingSurface()
@@ -1519,6 +1652,15 @@ public:
 
     virtual void DestroyEGLSurface(void)
     {
+#ifdef MOZ_WIDGET_GONK
+        mGraphicBuffer.clear();
+
+        if (mEGLImage) {
+            sEGLLibrary.fDestroyImage(EGL_DISPLAY(), mEGLImage);
+            mEGLImage = nsnull;
+        }
+#endif
+
         if (!mSurface)
             return;
 
@@ -1574,8 +1716,38 @@ public:
         }
 
         mBackingSurface = xsurface;
+
+        return mBackingSurface != nsnull;
 #endif
 
+#ifdef MOZ_WIDGET_GONK
+        if (gUseBackingSurface) {
+            mGLContext->MakeCurrent(true);
+            PixelFormat format = PixelFormatForImage(mUpdateFormat);
+            uint32_t usage = GraphicBuffer::USAGE_HW_TEXTURE |
+                             GraphicBuffer::USAGE_SW_READ_OFTEN |
+                             GraphicBuffer::USAGE_SW_WRITE_OFTEN;
+            mGraphicBuffer = new GraphicBuffer(aSize.width, aSize.height, format, usage);
+            if (mGraphicBuffer->initCheck() == OK) {
+                const int eglImageAttributes[] = { EGL_IMAGE_PRESERVED_KHR, LOCAL_EGL_TRUE,
+                                                   LOCAL_EGL_NONE, LOCAL_EGL_NONE };
+                mEGLImage = sEGLLibrary.fCreateImage(EGL_DISPLAY(),
+                                                        EGL_NO_CONTEXT,
+                                                        EGL_NATIVE_BUFFER_ANDROID,
+                                                        (EGLClientBuffer) mGraphicBuffer->getNativeBuffer(),
+                                                        eglImageAttributes);
+                if (!mEGLImage) {
+                    LOG("Could not create EGL images: ERROR (0x%04x)", sEGLLibrary.fGetError());
+                    return false;
+                }
+
+                return true;
+            }
+
+            LOG("GraphicBufferAllocator::alloc failed");
+            return false;
+        }
+#endif
         return mBackingSurface != nsnull;
     }
 
@@ -1586,12 +1758,16 @@ protected:
 
     nsIntRect mUpdateRect;
     ImageFormat mUpdateFormat;
+    bool mUsingDirectTexture;
     nsRefPtr<gfxASurface> mBackingSurface;
     nsRefPtr<gfxASurface> mUpdateSurface;
+#ifdef MOZ_WIDGET_GONK
+    sp<GraphicBuffer> mGraphicBuffer;
+#endif
+    EGLImage mEGLImage;
+    GLuint mTexture;
     EGLSurface mSurface;
     EGLConfig mConfig;
-    GLuint mTexture;
-    EGLImage mEGLImage;
     TextureState mTextureState;
 
     bool mBound;
@@ -1623,11 +1799,10 @@ GLContextEGL::TileGenFunc(const nsIntSize& aSize,
   GLuint texture;
   fGenTextures(1, &texture);
 
-  fActiveTexture(LOCAL_GL_TEXTURE0);
-  fBindTexture(LOCAL_GL_TEXTURE_2D, texture);
-
   nsRefPtr<TextureImageEGL> teximage =
       new TextureImageEGL(texture, aSize, LOCAL_GL_CLAMP_TO_EDGE, aContentType, this, aFlags);
+  
+  teximage->BindTexture(LOCAL_GL_TEXTURE0);
 
   GLint texfilter = aFlags & TextureImage::UseNearestFilter ? LOCAL_GL_NEAREST : LOCAL_GL_LINEAR;
   fTexParameteri(LOCAL_GL_TEXTURE_2D, LOCAL_GL_TEXTURE_MIN_FILTER, texfilter);
@@ -1666,6 +1841,14 @@ static const EGLint kEGLConfigAttribsRGB16[] = {
     LOCAL_EGL_NONE
 };
 
+static const EGLint kEGLConfigAttribsRGB24[] = {
+    LOCAL_EGL_SURFACE_TYPE,    LOCAL_EGL_WINDOW_BIT,
+    LOCAL_EGL_RENDERABLE_TYPE, LOCAL_EGL_OPENGL_ES2_BIT,
+    LOCAL_EGL_RED_SIZE,        8,
+    LOCAL_EGL_GREEN_SIZE,      8,
+    LOCAL_EGL_BLUE_SIZE,       8,
+    LOCAL_EGL_NONE
+};
 
 static const EGLint kEGLConfigAttribsRGBA32[] = {
     LOCAL_EGL_SURFACE_TYPE,    LOCAL_EGL_WINDOW_BIT,
@@ -1681,10 +1864,24 @@ static bool
 CreateConfig(EGLConfig* aConfig, PRInt32 depth)
 {
     EGLConfig configs[64];
-    const EGLint* attribs = depth == 16 ? kEGLConfigAttribsRGB16 :
-                                          kEGLConfigAttribsRGBA32;
+    const EGLint* attribs;
     EGLint ncfg = ArrayLength(configs);
 
+    switch (depth) {
+        case 16:
+            attribs = kEGLConfigAttribsRGB16;
+            break;
+        case 24:
+            attribs = kEGLConfigAttribsRGB24;
+            break;
+        case 32:
+            attribs = kEGLConfigAttribsRGBA32;
+            break;
+        default:
+            NS_ERROR("Unknown pixel depth");
+            return false;
+    }
+
     if (!sEGLLibrary.fChooseConfig(EGL_DISPLAY(), attribs,
                                    configs, ncfg, &ncfg) ||
         ncfg < 1) {
@@ -1704,7 +1901,8 @@ CreateConfig(EGLConfig* aConfig, PRInt32 depth)
             sEGLLibrary.fGetConfigAttrib(EGL_DISPLAY(), config,
                                          LOCAL_EGL_ALPHA_SIZE, &a) &&
             ((depth == 16 && r == 5 && g == 6 && b == 5) ||
-             (depth == 24 && r == 8 && g == 8 && b == 8 && a == 8)))
+             (depth == 24 && r == 8 && g == 8 && b == 8) ||
+             (depth == 32 && r == 8 && g == 8 && b == 8 && a == 8)))
         {
             *aConfig = config;
             return true;
@@ -1736,14 +1934,12 @@ CreateConfig(EGLConfig* aConfig)
     }
 }
 
+// When MOZ_JAVA_COMPOSITOR is defined,
+// use mozilla::AndroidBridge::Bridge()->ProvideEGLSurface() instead.
+#ifndef MOZ_JAVA_COMPOSITOR
 static EGLSurface
 CreateSurfaceForWindow(nsIWidget *aWidget, EGLConfig config)
 {
-#ifdef MOZ_JAVA_COMPOSITOR
-    // Use mozilla::AndroidBridge::Bridge()->ProvideEGLSurface() instead.
-    NS_RUNTIMEABORT("CreateSurfaceForWindow should not be called on Native Fennec.");
-#endif
-
     EGLSurface surface;
 
 #ifdef DEBUG
@@ -1778,6 +1974,7 @@ CreateSurfaceForWindow(nsIWidget *aWidget, EGLConfig config)
 
     return surface;
 }
+#endif
 
 already_AddRefed<GLContext>
 GLContextProviderEGL::CreateForWindow(nsIWidget *aWidget)
@@ -1788,11 +1985,7 @@ GLContextProviderEGL::CreateForWindow(nsIWidget *aWidget)
         return nsnull;
     }
 
-#if defined(XP_WIN) || defined(ANDROID) || defined(MOZ_PLATFORM_MAEMO)
     bool doubleBuffered = true;
-#else
-    bool doubleBuffered = false;
-#endif
 
     void* currentContext = sEGLLibrary.fGetCurrentContext();
     if (aWidget->HasGLContext() && currentContext) {
@@ -1817,6 +2010,9 @@ GLContextProviderEGL::CreateForWindow(nsIWidget *aWidget)
         if (!glContext->Init())
             return nsnull;
 
+        glContext->MakeCurrent();
+        sEGLLibrary.LoadConfigSensitiveSymbols();
+
         glContext->SetIsDoubleBuffered(doubleBuffered);
 
         glContext->SetPlatformContext(platformContext);
@@ -1861,6 +2057,8 @@ GLContextProviderEGL::CreateForWindow(nsIWidget *aWidget)
         return nsnull;
     }
 
+    glContext->MakeCurrent();
+    sEGLLibrary.LoadConfigSensitiveSymbols();
     glContext->SetIsDoubleBuffered(doubleBuffered);
 
     return glContext.forget();
@@ -2294,10 +2492,19 @@ GLContextProviderEGL::CreateForNativePixmapSurface(gfxASurface* aSurface)
 GLContext *
 GLContextProviderEGL::GetGlobalContext(const ContextFlags)
 {
-#ifdef ANDROID
+// Don't want a global context on Android as 1) share groups across 2 threads fail on many Tegra drivers (bug 759225)
+// and 2) some mobile devices have a very strict limit on global number of GL contexts (bug 754257)
+#ifdef MOZ_JAVA_COMPOSITOR
     return nsnull;
 #endif
 
+// Don't want a global context on Windows as we don't use it for WebGL surface sharing and it makes
+// context creation fail after a context loss (bug 764578)
+#ifdef XP_WIN
+    return nsnull;
+#endif
+
+
     static bool triedToCreateContext = false;
     if (!triedToCreateContext && !gGlobalContext) {
         triedToCreateContext = true;
