@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /*
  * implementation of interface that allows layout-debug extension access
@@ -49,7 +17,7 @@
 
 using namespace mozilla::layers;
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
 class nsLayoutDebugger : public nsILayoutDebugger {
 public:
   nsLayoutDebugger();
@@ -148,23 +116,30 @@ nsLayoutDebugger::GetStyleSize(nsIPresShell* aPresentation,
   *aSizeInBytesResult = 0;
   return NS_ERROR_FAILURE;
 }
+#endif
 
+#ifdef MOZ_DUMP_PAINTING
 static void
 PrintDisplayListTo(nsDisplayListBuilder* aBuilder, const nsDisplayList& aList,
-                   PRInt32 aIndent, FILE* aOutput)
+                   FILE* aOutput)
 {
+  fprintf(aOutput, "<ul>");
+
   for (nsDisplayItem* i = aList.GetBottom(); i != nsnull; i = i->GetAbove()) {
+#ifdef DEBUG
     if (aList.DidComputeVisibility() && i->GetVisibleRect().IsEmpty())
       continue;
-    for (PRInt32 j = 0; j < aIndent; ++j) {
-      fputc(' ', aOutput);
-    }
+#endif
+    fprintf(aOutput, "<li>");
     nsIFrame* f = i->GetUnderlyingFrame();
     nsAutoString fName;
+#ifdef DEBUG
     if (f) {
       f->GetFrameName(fName);
     }
-    nsRect rect = i->GetBounds(aBuilder);
+#endif
+    bool snap;
+    nsRect rect = i->GetBounds(aBuilder, &snap);
     switch (i->GetType()) {
       case nsDisplayItem::TYPE_CLIP:
       case nsDisplayItem::TYPE_CLIP_ROUNDED_RECT: {
@@ -183,8 +158,16 @@ PrintDisplayListTo(nsDisplayListBuilder* aBuilder, const nsDisplayList& aList,
         nsDisplayTransform* t = static_cast<nsDisplayTransform*>(i);
         list = t->GetStoredList()->GetList();
     }
+#ifdef DEBUG
     if (!list || list->DidComputeVisibility()) {
-      opaque = i->GetOpaqueRegion(aBuilder);
+      opaque = i->GetOpaqueRegion(aBuilder, &snap);
+    }
+#endif
+    if (i->Painted()) {
+      nsCString string(i->Name());
+      string.Append("-");
+      string.AppendInt((PRUint64)i);
+      fprintf(aOutput, "<a href=\"javascript:ViewImage('%s')\">", string.BeginReading());
     }
     fprintf(aOutput, "%s %p(%s) (%d,%d,%d,%d)(%d,%d,%d,%d)%s%s",
             i->Name(), (void*)f, NS_ConvertUTF16toUTF8(fName).get(),
@@ -192,25 +175,35 @@ PrintDisplayListTo(nsDisplayListBuilder* aBuilder, const nsDisplayList& aList,
             vis.x, vis.y, vis.width, vis.height,
             opaque.IsEmpty() ? "" : " opaque",
             i->IsUniform(aBuilder, &color) ? " uniform" : "");
+    if (i->Painted()) {
+      fprintf(aOutput, "</a>");
+    }
     if (f) {
       PRUint32 key = i->GetPerFrameKey();
       Layer* layer = aBuilder->LayerBuilder()->GetOldLayerFor(f, key);
       if (layer) {
-        fprintf(aOutput, " layer=%p", layer);
+        fprintf(aOutput, " <a href=\"#%p\">layer=%p</a>", layer, layer);
       }
     }
+    if (i->GetType() == nsDisplayItem::TYPE_SVG_EFFECTS) {
+      (static_cast<nsDisplaySVGEffects*>(i))->PrintEffects(aOutput);
+    }
     fputc('\n', aOutput);
     if (list) {
-      PrintDisplayListTo(aBuilder, *list, aIndent + 4, aOutput);
+      PrintDisplayListTo(aBuilder, *list, aOutput);
     }
+    fprintf(aOutput, "</li>");
   }
+  
+  fprintf(aOutput, "</ul>");
 }
 
 void
 nsFrame::PrintDisplayList(nsDisplayListBuilder* aBuilder,
-                          const nsDisplayList& aList)
+                          const nsDisplayList& aList,
+                          FILE* aFile)
 {
-  PrintDisplayListTo(aBuilder, aList, 0, stdout);
+  PrintDisplayListTo(aBuilder, aList, aFile);
 }
 
 #endif
