@@ -11,6 +11,7 @@
 #include "nsIComponentManager.h"
 #include "nsGfxCIID.h"
 #include "nsIInterfaceRequestor.h"
+#include "mozilla/Attributes.h"
 
 //mmptemp
 
@@ -26,7 +27,7 @@ static nsEventStatus HandleEvent(nsGUIEvent *aEvent);
 /**
  * nsISupports-derived helper class that allows to store and get a view
  */
-class ViewWrapper : public nsIInterfaceRequestor
+class ViewWrapper MOZ_FINAL : public nsIInterfaceRequestor
 {
   public:
     NS_DECLARE_STATIC_IID_ACCESSOR(VIEW_WRAPPER_IID)
@@ -529,7 +530,7 @@ void nsView::InsertChild(nsView *aChild, nsView *aSibling)
   {
     if (nsnull != aSibling)
     {
-#ifdef NS_DEBUG
+#ifdef DEBUG
       NS_ASSERTION(aSibling->GetParent() == this, "tried to insert view with invalid sibling");
 #endif
       //insert after sibling
