@@ -6,11 +6,12 @@
 #include "nsIMemoryReporter.h"
 #include "nsCOMArray.h"
 #include "mozilla/Mutex.h"
+#include "mozilla/Attributes.h"
 #include "nsString.h"
 
 using mozilla::Mutex;
 
-class nsMemoryReporter : public nsIMemoryReporter
+class nsMemoryReporter MOZ_FINAL : public nsIMemoryReporter
 {
 public:
   NS_DECL_ISUPPORTS
